/*
 * main.c
 * Vincas Dargis 2010 <vindrg@gmail.com>
 * 
 * kava-info-applet is Public Domain software.
 * 
 * kava-info-applet is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <panel-applet.h>
#include <libsoup/soup.h>
#include <errno.h>

#define KAVA_PROGRAM_NAME "Kava Info Applet"

#define KAVA_PROGRAM_VERSION "1.0.0"

#define KAVA_LOGO_24_PNG "/usr/share/pixmaps/kava-info_24x24.png"
#define KAVA_LOGO_48_PNG "/usr/share/pixmaps/kava-info_48x48.png"

//! data fetching interval in seconds
#define KAVA_TIMER_INTERVAL 60 * 10

//! seconds to wait for http request to finish.
#define KAVA_HTTP_REQUEST_TIMEOUT 10

//! critical limit in MB ( less or equal ), to show in red color.
#define KAVA_LIMIT_CRITICAL 100

//! HTTP request uri
#define KAVA_HTTP_REQUEST_URI "http://www.kava.lt/Statistika.php"

//! Format for showing ivalid statistics (first launch, on error, etc.)
#define KAVA_FORMAT_UNDEFINED "? MB"

//! Format for showing valid statistics information.
#define KAVA_FORMAT_VALID "%d MB"

#define KAVA_FORMAT_ERROR "<span color='red'>%d MB</span>"

//TODO: not used anymore, remove..?
//! Perl regex to find how many MB is total available.
#define KAVA_REGEXP_TOTAL "(?<=Duomenų kiekis maksimaliu greičiu:<b>\\s)\\d+"

//! Perl regex to find how many MB is left.
#define KAVA_REGEXP_LEFT "(?<=Jūs galite priimti\\s)\\d+"

//! Got ideas how to make it in more "proper" way? Email me.
#define KAVA_FAKE_TRANSLATION_0 N_("_Refresh")
#define KAVA_FAKE_TRANSLATION_1 N_("_About...")
#define KAVA_FAKE_TRANSLATION_2 N_("Credits")

static const gchar* kava_authors[] = { 
	"Vincas Dargis <vindrg@gmail.com>", 
	NULL 
};

static GtkWidget* kava_label = NULL;

static const char kava_menu_xml [] =
	"<popup name=\"button3\">\n"
	"   <menuitem name=\"Refresh Item\" "
	"             verb=\"KavaRefresh\" "
	"           _label=\"_Refresh\"\n"
	"          pixtype=\"stock\" "
	"          pixname=\"gnome-stock-refresh\"/>\n"
	"   <menuitem name=\"About Item\" "
	"             verb=\"KavaAbout\" "
	"           _label=\"_About...\"\n"
	"          pixtype=\"stock\" "
	"          pixname=\"gnome-stock-about\"/>\n"
	"</popup>\n";

static gboolean kava_on_timer(gpointer data);

struct MultiRes;

static void kava_menu_about(BonoboUIComponent * component, struct MultiRes* applet)
{
	GtkWidget* about = gtk_about_dialog_new();

	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(about), KAVA_PROGRAM_NAME);
	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), KAVA_PROGRAM_VERSION);
	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(about), kava_authors);
	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about), _("KAVA Internet Service Provider account statistics fetcher."));
		
	gtk_dialog_run(GTK_DIALOG(about));
	gtk_widget_destroy(about);
}

static void kava_menu_refresh(BonoboUIComponent * component, struct MultiRes* applet)
{
	kava_on_timer(NULL);
}

static const BonoboUIVerb kava_menu_verbs [] = {
	BONOBO_UI_VERB("KavaAbout", (BonoboUIVerbFn)kava_menu_about),
	BONOBO_UI_VERB("KavaRefresh", (BonoboUIVerbFn)kava_menu_refresh),
	BONOBO_UI_VERB_END
};

//! Search and parse number value
static gboolean kava_parse_num(const gchar* string, gssize length, const gchar* expresion, int* result)
{
	gboolean res = TRUE;
	GRegex* rx = g_regex_new(expresion, 0, 0, NULL);
	GMatchInfo* match = NULL;

	g_regex_match_full(rx, string, length, 0, 0, &match, NULL);

	if((res = g_match_info_matches(match))) {
		gchar *word = g_match_info_fetch(match, 0);
		fprintf(stderr, "Found: %s\n", word);
		*result = (int)g_ascii_strtoll(word, NULL, 10);
		g_free(word);
	}

	g_match_info_free(match);
	g_regex_unref(rx);

	return res;
}


//! Downloads ISP account statistics. False on HTTP error.
static gboolean kava_fetch_statistics(int* left)
{
	SoupSession* session = soup_session_sync_new_with_options(SOUP_SESSION_TIMEOUT, KAVA_HTTP_REQUEST_TIMEOUT, NULL);
	SoupMessage* message = soup_message_new("GET", KAVA_HTTP_REQUEST_URI); 

	fprintf(stderr, "kava-info-applet: sending http request... ");
	const guint status = soup_session_send_message(session, message);
	fprintf(stderr, "done, status = %d\n", status);

	gboolean result = FALSE;

	if(status == SOUP_STATUS_OK) {
		//it's considered OK if http response is valid, even if could not find/parse value (it will happen when all MB's are consumed).
		result = TRUE;
		fprintf(stderr, "kava-info-applet: response data (length = %d): \n", (int)message->response_body->length);
		kava_parse_num((gchar*)message->response_body->data, (gssize)message->response_body->length, KAVA_REGEXP_LEFT, left);
	} else {
		fprintf(stderr, "kava-info-applet: got invalid response, original message: %s\n", message->reason_phrase);
	}

	g_object_unref(message);
	g_object_unref(session);
	
    	return result;
}

//! Timer function.
static gboolean kava_on_timer(gpointer data)
{
	int left = 0;

	gchar* t = NULL;

	if(!kava_fetch_statistics(&left)) {
		t = g_strdup(KAVA_FORMAT_UNDEFINED);
	} else if(left <= KAVA_LIMIT_CRITICAL) {
		t = g_strdup_printf(KAVA_FORMAT_ERROR, left);
	} else {
		t = g_strdup_printf(KAVA_FORMAT_VALID, left);
	}

	gtk_label_set_markup(GTK_LABEL(kava_label), t);
	g_free(t);    

	// timer will continue.	
	return TRUE;
}

//! Factory function; creates applet with widgets and starts downloading ISP account statistics for the first time.
static gboolean kava_applet_factory(PanelApplet *applet, const gchar *iid, gpointer user_data)
{
	//Only one instance is allowed
	static gboolean guard = FALSE;
	GtkWidget *box;
	GtkWidget *image;

	//allow only one instance.
	if(guard || strcmp(iid, "OAFIID:kava-info-appletApplet") != 0)
		return FALSE;

	guard = TRUE;

	box = gtk_hbox_new(FALSE, 0);
	kava_label = gtk_label_new(NULL);

	gtk_container_add(GTK_CONTAINER(box), kava_label);

	image = gtk_image_new_from_file(KAVA_LOGO_24_PNG);

	gtk_container_add(GTK_CONTAINER(box), image);    
	gtk_container_add(GTK_CONTAINER(applet), box);    
	gtk_widget_show_all(GTK_WIDGET(applet));

	g_timeout_add_seconds(KAVA_TIMER_INTERVAL, kava_on_timer, NULL);

	panel_applet_setup_menu(PANEL_APPLET(applet), kava_menu_xml, kava_menu_verbs, NULL);	 
	gtk_window_set_default_icon_from_file(KAVA_LOGO_48_PNG, NULL);

	fprintf(stderr, "kava-info-applet: initialized.\n");

	//initial data fetch.
	kava_on_timer(NULL);

	return TRUE;
}

PANEL_APPLET_BONOBO_FACTORY("OAFIID:kava-info-appletApplet_Factory",
                             PANEL_TYPE_APPLET,
                             "Kava Info Applet",
                             "0",
                             kava_applet_factory,
                             NULL);

